/*
 * JBox2D - A Java Port of Erin Catto's Box2D
 * 
 * JBox2D homepage: http://jbox2d.sourceforge.net/ 
 * Box2D homepage: http://www.gphysics.com
 * 
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software
 * in a product, an acknowledgment in the product documentation would be
 * appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */
package org.jbox2d.testbed.tests;

import java.util.ArrayList;
import java.util.List;

import org.jbox2d.collision.CircleDef;
import org.jbox2d.collision.PolygonDef;
import org.jbox2d.common.Vec2;
import org.jbox2d.dynamics.Body;
import org.jbox2d.dynamics.BodyDef;
import org.jbox2d.dynamics.joints.MouseJoint;
import org.jbox2d.dynamics.joints.MouseJointDef;
import org.jbox2d.testbed.AbstractExample;
import org.jbox2d.testbed.TestbedMain;

public class Smash extends AbstractExample {

	int numlevels = 4;
	
	Body smasher ;
	List<Body> bricks ;//= new ArrayList<Body>();
	
    public Smash(TestbedMain p) {
        super(p);
    }
    
    public String getName() {
    	return "Smash";
    }

    @Override
    public void create() {
        //floor
            PolygonDef sd = new PolygonDef();
            sd.setAsBox(50.0f, 10.0f);

            BodyDef bd = new BodyDef();
            bd.position = new Vec2(0.0f, -20.0f);
            m_world.createBody(bd).createShape(sd);
            
            
            makePlatform();
            
      

        {


            makeBricks();

            CircleDef cd = new CircleDef();
            cd.density = 5.0f;
            cd.friction = 0.3f;
            cd.restitution = 0.0f;
            //sd.setAsBox(3.0f, 3.0f);
            
            
            bd = new BodyDef();
            Vec2 startingMousePos = m_debugDraw.screenToWorld(100,100); 
            bd.position = startingMousePos;
            //new Vec2(    new Vec2(mouseX,mouseY));
            
            smasher = m_world.createBody(bd);
            smasher.createShape(cd);
            smasher.setMassFromShapes();
            
            MouseJointDef md = new MouseJointDef();
            md.body1 = m_world.getGroundBody();
            md.body2 = smasher;
            md.target.set(startingMousePos);
            md.maxForce = 150.0f * smasher.m_mass;
            m_mouseJoint = (MouseJoint) m_world.createJoint(md);
           // body.wakeUp();
            
            
        }
        
     
    }
    public String getExampleInstructions() {
		return "SMASHENFREUDE - the joy of smashing\nknock all blocks below platform - space restarts - scrollwheel zooms\n";
	}
	public void postStep() {
		//m_debugDraw.worldToScreen
	
		
		if(newKeyDown[' ']==true){
			numlevels = 4;
			makeBricks();
		}
		int numup = 0;
		int numdown = 0;
		for(Body brick: bricks){
			Vec2 loc = brick.getWorldCenter();
			
			
			if(loc.y >= PLATFORM_TOP){
				numup++;
			} else {
				numdown++;
			}

		}
		if(numup == 0){
			numlevels++;

			makeBricks();
		}
		//System.out.println("numup "+numup+" nuymdown "+numdown);
		
		
		
	}
    
    public void mouseMove(Vec2 p) {
    	mouseScreen.set(p);
        if (m_mouseJoint != null) {
            m_mouseJoint.setTarget(m_debugDraw.screenToWorld(p));
        }
    }
    
    @Override
    public void mouseDown(Vec2 p) {
    //	m_world.destroyBody(smasher);
    	
    }
    @Override
    public void mouseUp(){
    	
    }
    
    
    
    void makeBricks(){
        PolygonDef sd = new PolygonDef();

        sd.setAsBox(BRICKSIZE,BRICKSIZE);
        sd.density = 1.0f;
        sd.friction = 0.3f;
        sd.restitution = 0.0f;
        BodyDef bd = new BodyDef();

        if(bricks != null){
        	for(Body brick:bricks){
        		m_world.destroyBody(brick);
        	}
        	
        }
        
        bricks = new ArrayList<Body>();
        for (int i = 1; i <= numlevels; i++) {
        	int numforrow = 1 + ((numlevels - i));
        	float left = -1.0f * (numforrow ) ;
        	for(int j = 0; j < numforrow;j++){

            bd.position = new Vec2(left + (2*j*BRICKSIZE ),PLATFORM_TOP + (2*i*BRICKSIZE ));
            Body brickBody = m_world.createBody(bd);
            sd.setAsBox((float)BRICKSIZE, (float)(BRICKSIZE*(.2+Math.random())*2));
            brickBody.createShape(sd);
            brickBody.setMassFromShapes();
            bricks.add(brickBody);
        	}
        }
    }
    float BRICKSIZE = 1.0f;
    float PLATFORM_HEIGHT = 1.0f;
    float PLATFORM_Y = 5.0f;
    float PLATFORM_TOP = (PLATFORM_HEIGHT /2)+ PLATFORM_Y ; 
    void makePlatform(){
    	PolygonDef sd = new PolygonDef();
    	sd.setAsBox(15.0f, PLATFORM_HEIGHT);
    	BodyDef bd = new BodyDef();
    	bd.position = new Vec2(0.0f, PLATFORM_Y);
    	m_world.createBody(bd).createShape(sd);
    }
    

    
    
}